#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include <string.h>
#include "gpio.h"
#define USE_SERIAL Serial

#define PI 3.1415926535897932384626433832795

//WebSocket
const int port = 2010;
const char* host = "10.1.1.12";

//WEB Server
ESP8266WebServer server;

//APҦSSIDKX
const char* softap_ssid = "pa_ver1_003";
const char* softap_pass = "123456789";

//qEEPROMŪXӪSSIDKX
String eeprom_ssid;
String eeprom_pass;

//IPm
String AP_IP;
String STA_IP;

//UUID
uint8_t MAC_ADDR[6];
char UUID[18];

//reboot flag
int reboot_flag = 0;
int reboot_try = 0;
int status = WL_IDLE_STATUS;
bool is_epass = false;

int Input_1 = 13;

int iv1 = 0;
int iv2 = 0;
int ia1 = 0;
int ia2 = 0;

WiFiClient client;

void setup() {
	//Serial.begin(9600);
	//while (!Serial){}
	
	is_epass = is_epassf();
	read_ep_wifi();		//ŪEEPROMSTAbKX
	get_uuid();			//o˸mUUID
	ESP.wdtDisable();
	ESP.wdtEnable(15000);
	
	Serial.printf("\r\n[BOOT] success.");
	pinMode(Input_1, INPUT_PULLUP);
	if(digitalRead(Input_1) == LOW) {
		//iJCONFIG MODE
		//byte send_btn[3] = { 0xAB, 0xAB, 0xAC };
		//Serial.write(send_btn, 3);
		setupAP();
		while(1) {
			server.handleClient();
		}
	} else {
		WiFi.mode(WIFI_STA);
		set_sta_wifi();
	}
	
	if (!client.connect(host, port)) {
		Serial.println("connection failed");
    }
}

void loop() {
	if(client.connected()){
		int dlen = 200;
		int plen = dlen * 2 + 15;
		char HDATA[415] = {
			0xf0, 0xfc, 0xff, 0xff,
			MAC_ADDR[0], MAC_ADDR[1], MAC_ADDR[2], MAC_ADDR[3], MAC_ADDR[4], MAC_ADDR[5],
			0xff, 0xff, 0xff, 0xff
		};
		HDATA[14] = dlen & 0xff;
		for(int icnt=0; icnt<dlen; icnt++) {
			int rand_num = ((sin(2*PI*icnt/50)*60) + (0.2*(sin(2*PI*icnt*3/50)*180)) + (0.1*(sin(2*PI*icnt*5/50)*300))) + 500;
			//int rand_num = icnt + 500;
			HDATA[icnt*2 + 15] = (rand_num > 8) & 0xff;
			HDATA[icnt*2 + 16] = rand_num & 0xff;
		}
		client.print(HDATA);
	} else {
		client.connect(host, port);
	}
}

bool is_epassf() {
	if(EEPROM.read(100) == 0x00){		//wg]w
		return true;
	}
	return false;
}
bool set_sta_wifi(void) {
	if(reboot_try > 10){
		ESP.reset();
	}
	if ( eeprom_ssid.length() > 1) {
		eeprom_ssid = eeprom_ssid.c_str();
		eeprom_pass = eeprom_pass.c_str();
		WiFi.disconnect();
		WiFi.begin(eeprom_ssid.c_str(), eeprom_pass.c_str());
		int y = 0;
		while(y < 100){
			y++;
			server.handleClient();
			delay(100);
			if (WiFi.status() == WL_CONNECTED) {
				//Serial.print("\r\n[STA] connect success.");
				reboot_try = 0;
				reboot_flag = 0;
				return true;
			}
		}
	}
	reboot_try++;
	//Serial.printf("\r\n[STA] connect fail(%d).", reboot_try);
	return false;
}

void read_ep_wifi(void) {
	EEPROM.begin(512);
	delay(10);
	eeprom_ssid = "";
	for (int i = 0; i < 32; ++i) {
		if(EEPROM.read(i) == 0x00){
			break;
		}
		eeprom_ssid += char(EEPROM.read(i));
	}
	eeprom_pass = "";
	for (int i = 32; i < 96; ++i) {
		if(EEPROM.read(i) == 0x00){
			break;
		}
		eeprom_pass += char(EEPROM.read(i));
	}
	delay(10);
	eeprom_ssid.trim();
	eeprom_pass.trim();
}

void get_uuid(void) {
	// o˸mSSID
	WiFi.macAddress(MAC_ADDR);
	for (int i = 0; i < sizeof(MAC_ADDR); ++i) {
		sprintf(UUID, "%s%02X", UUID, MAC_ADDR[i]);
	}
}

void renew_ip_addr(void) {
	//AP
	IPAddress AP_IP_IA = WiFi.softAPIP();
	AP_IP = String(AP_IP_IA[0]) + '.' + String(AP_IP_IA[1]) + '.' + String(AP_IP_IA[2]) + '.' + String(AP_IP_IA[3]);
	if(AP_IP != "0.0.0.0") {
		//Serial.printf("\r\n[AP] connected, ip = %s", AP_IP.c_str());
	} else {
		Serial.print("\r\n[AP] not connect.");
	}
	
	//STA
	IPAddress STA_IP_IA = WiFi.localIP();
	STA_IP = String(STA_IP_IA[0]) + '.' + String(STA_IP_IA[1]) + '.' + String(STA_IP_IA[2]) + '.' + String(STA_IP_IA[3]);
	if(STA_IP != "0.0.0.0") {
		//Serial.printf("\r\n[STA] connected, ip = %s", STA_IP.c_str());
	} else {
		Serial.print("\r\n[STA] not connect.");
	}
}

void launchWeb(void) {
	renew_ip_addr();
	createWebServer();
	server.begin();
}

void setupAP(void) {
	WiFi.mode(WIFI_AP);
	//}APҦ - Default:WPA2
	WiFi.softAP(softap_ssid, softap_pass);
	IPAddress ip = WiFi.localIP();
	server = ESP8266WebServer(ip, 80);
	launchWeb();
}

void createWebServer(void) {
	server.on("/wifi_list", []() {
		int n = WiFi.scanNetworks();
		String content = "";
		content += "{";
		for (int i = 0; i < n; ++i) {
			// Print SSID and RSSI for each network found
			content += "{'ssid':";
			content += WiFi.SSID(i);
			content += ", 'rssi':";
			content += WiFi.RSSI(i);
			content += ", 'encrypt':";
			content += WiFi.encryptionType(i);
			content += "}\r\n";
		}
		content += "}";
		server.send(200, "text/plain", content);
	});
	server.on("/style.css", []() {
		String content = "a{text-decoration: none;}\r\n";
		content += "table{border-collapse: collapse;}\r\n";
		content += "table,th,td{border: 1px solid #000;}\r\n";
		content += "p{margin:10px !important;font-size:12px;}\r\n";
		content += ".input_text{width:120px;}\r\n";
		server.send(200, "text/css", content);
	});
	server.on("/", []() {
		//sIP
		renew_ip_addr();
		//read_ep_wifi();
		//]w
		String content =	"<!DOCTYPE HTML>\r\n";
		content += "<html>\r\n";
		content += "	<head>\r\n";
		content += "		<meta http-equiv='Content-Type' content='text/html;charset=UTF-8'/>\r\n";
		content += "		<meta http-equiv='Cache-Control' content='no-cache'/>\r\n";
		content += "		<meta http-equiv='X-UA-Compatible' content='IE=edge'>\r\n";
		content += "		<meta name='viewport' content='width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no'/>\r\n";
		content += "		<link href='./style.css' rel='stylesheet'>\r\n";
		content += "		<title>su]w</title>\r\n";
		content += "	</head>\r\n";
		content += "	<body>\r\n";
		content += "		<form method='get' action='setting' style='padding-top:20px;'>\r\n";
		content += "			<table align='center' style='max-width:450px;' width='90%'>\r\n";
		content += "				<tr bgcolor='#fcc'>\r\n";
		content += "					<td colspan='2' align='center'><h2>su]w</h2></td>\r\n";
		content += "				</tr>\r\n";
		content += "				<tr bgcolor='#fcc'>\r\n";
		content += "					<td align='right'><p>AP IP</p></td>\r\n";
		content += "					<td align='left'>\r\n";
		content += "						<p>" + AP_IP + "</p>\r\n";
		content += "					</td>\r\n";
		content += "				</tr>\r\n";
		content += "				<tr bgcolor='#fcc'>\r\n";
		content += "					<td align='right'><p>STA IP</p></td>\r\n";
		content += "					<td align='left'>\r\n";
		content += "						<p>" + STA_IP + "</p>\r\n";
		content += "					</td>\r\n";
		content += "				</tr>\r\n";
		content += "				<tr>\r\n";
		content += "					<td align='right'><p>WIFI b</p></td>\r\n";
		content += "					<td align='left'><p><input class='input_text' name='ssid' type='input' value='" + eeprom_ssid + "'></p></td>\r\n";
		content += "				</tr>\r\n";
		content += "				<tr>\r\n";
		content += "					<td align='right'><p>WIFI KX</p></td>\r\n";
		content += "					<td align='left'><p><input class='input_text' name='pass' type='input' value='" + eeprom_pass + "'></p></td>\r\n";
		content += "				</tr>\r\n";
		content += "				<tr bgcolor='#fcc'>\r\n";
		content += "					<td colspan='2' align='center'>\r\n";
		content += "						<p><input type='submit' value='xs]w'></p>\r\n";
		if(reboot_flag == 1){
			content += "						<p style='color:red'>]wwܡA}ͮ</p>\r\n";
		}
		content += "					</td>\r\n";
		content += "				</tr>\r\n";
		content += "			</table>\r\n";
		content += "		</form>\r\n";
		content += "	</body>\r\n";
		content += "</html>";
		server.send(200, "text/html", content);
	});
	server.on("/setting", []() {
		//xse
		String content =	"<!DOCTYPE HTML>\r\n";
		content += "<html>\r\n";
		content += "	<head>\r\n";
		content += "		<meta http-equiv='Content-Type' content='text/html;charset=UTF-8'/>\r\n";
		content += "		<meta http-equiv='Cache-Control' content='no-cache'/>\r\n";
		content += "		<meta http-equiv='X-UA-Compatible' content='IE=edge'>\r\n";
		content += "		<meta name='viewport' content='width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no'/>\r\n";
		content += "		<link href='./style.css' rel='stylesheet'>\r\n";
		content += "		<title>su]w</title>\r\n";
		content += "	</head>\r\n";
		content += "	<body>\r\n";
		content += "		<p style='text-align:center;font-size:12px;color:red;'>]wwxsAé󭫶}ͮġAФʭ}C <a href='/'>[^]w]</a></p>\r\n";
		content += "		<script type='text/javascript'>alert(']wwxsAé󭫶}ͮġAЭs}C');setTimeout(\"window.location='http://google.com'\",5000);</script>\r\n";
		content += "	</body>\r\n";
		content += "</html>";
		server.send(200, "text/html", content);
		//xsEEPROM
		String qsid = server.arg("ssid");
		String qpass = server.arg("pass");
		eeprom_ssid = qsid;
		eeprom_pass = qpass;
		if (qsid.length() > 0 && qpass.length() > 0) {
			EEPROM.write(100, 0x01);	//flag
			for (int i = 0; i < 96; ++i) {
				EEPROM.write(i, 0);
			}
			for (int i = 0; i < qsid.length(); ++i) {
				EEPROM.write(i, qsid[i]);
			}
			for (int i = 0; i < qpass.length(); ++i) {
				EEPROM.write(32 + i, qpass[i]);
			}
			EEPROM.write(100, 0x00);	//flag
			EEPROM.commit();
			is_epass = true;
		}
		//Serial.printf("\r\n[WEB] save setting success.");
		reboot_flag = 1;
		//s}
		ESP.reset();
		set_sta_wifi();
	});
}